/*
 * Copyright (C) 2005  ENSI-GAME-ART
 *
 * Club ENSI-GAME-ART ENSICAEN
 * 6 Boulevard Marechal Juin
 * F-14050 Caen Cedex
 *
 * Ce fichier est l'oeuvre d'eleves de l'ENSI de Caen. Il est developp
 * sous la licence GPL :
 *
 ****************************************************************
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *****************************************************************
 *
 * Ce fichier fait partie du projet de jeu video de course spatiale
 * developpe au sein du club ENSI-GAME-ART
 */

#include "display/Ega_SystemParticule.h"

#include "other/Ega_debug.h"

/////////////////////////////////////////////   FONCTIONS GENERALES   //////////////////////////////////////////////////



////////////////////////////////////////////////////
// Constructeur initialise le systeme selon
//	le type et le npombre nbre de particules a gerer
////////////////////////////////////////////////////

Ega_SystemParticule :: Ega_SystemParticule(int nbre,int type)
{
	int i;

	/*  inits */
	SYSnbre			= nbre;													// nbre de particules a gerer
	SYStype			= type;													// type de systeme
	SYSparticules	= (Particule *)malloc(SYSnbre*sizeof(Particule));		// allocation ressources tableau de particules
	SYStaille		= 10.0;													// taille par defaut
	SYSpas			= 1.0;													// pas de vieillissement par defaut
	SYSvitesse		= 1.0;													// vitesse de deplacement des particules par defaut
	SYSmat			= NULL;													// texture par defaut

	/* chaque particule active */
	for(i=0;i<SYSnbre;i++)
	{
		SYSparticules[i].active = true;
	}

	/* duree de vie par defaut */ 
	DefinirDureeDeVie(500.0,0.0);

	/* attraction par defaut */
	DefinirAttraction(0.0,0.0,0.0);

/* selon le type de systeme faire init qui va */

	if((SYStype & 0x0000000F) == REACTEUR)			// REACTEUR
	{
		SYSmet = NULL;								// pas de METEO
		SYSreact = new Reacteur;					// init structure

		/* inits REACTEUR */
		InitReacteur(0.0,0.0,0.0,10.0);				// positionnne source en 0,0,0 de diametre 10
		OrienterReacteur(0.0,1.0,0.0,0.5,0.5,0.5);	// oriente le reacteur  vers l'axe y positif
		DefinirToleranceReacteur(0.0,0.0,0.0);		// pas de tolerance
	}

	else if((SYStype & 0x0000000F) == METEO)		// METEO
	{
		SYSreact = NULL;							// pas de REACTEUR
		SYSmet = new Meteo;							// init structure
		InitMeteo(0.5,0.2,false);
		InitZoneMeteo(0.0,0.0,0.0,20.0,20.0,20.0);
	}

	else											// inconnu
	{
		PRINT_PARTICULE1("Constructeur : Type de syteme inexistant !\n");		// erreur
	}

/* selon le mode de couleur */

	if((SYStype & 0x000000F0) == UNIQUE)			// UNIQUE
	{
		SYSarc = NULL;
		SYSmor = NULL;
		DefinirCouleurUnique(1.0,1.0,1.0);	// couleur unique par defaut : blanc
	}

	else if((SYStype & 0x000000F0) == MORPH)		// MORPH
	{
		SYSmor = new Morph;
		SYSarc = NULL;
		InitMorph(SYSviemax/2,true);
		DefinirCouleurDepart(1.0,0.0,0.0);
		DefinirCouleurArrivee(1.0,1.0,0.0);
	}

	else if((SYStype & 0x000000F0) == ALEATOIRE)	// ALEATOIRE
	{
		SYSarc = NULL;
		SYSmor = NULL;
		DefinirCouleurs(0,NULL);
	}

	else if((SYStype & 0x000000F0) == ARC_EN_CIEL)	// ARC_EN_CIEL
	{
		SYSarc = new ArcEnCiel;
		SYSmor = NULL;
		//DefinirCouleursArc();
	}



	else											// inconnu
	{
		PRINT_PARTICULE1("Constructeur : Modele de couleurs inexistant !\n");		// erreur
	}
}


////////////////////////////////////////////////////
// DefinirDureeDeVie definit un intervalle dans lequel
//	elle va choisir aleatoirement une duree de vie
//	pour chacune des particules
////////////////////////////////////////////////////

void Ega_SystemParticule :: DefinirDureeDeVie(float viemax,float viemin)
{
	int i;
	SYSviemax = viemax;				// sauvegarde bornes
	SYSviemin = viemin;
	srand(time(NULL));				// init random
	for(i=0;i<SYSnbre;i++)			// pour chaque particule
	{
		SYSparticules[i].vie = SYSviemin + (((float)rand()/RAND_MAX)*(SYSviemax-SYSviemin));	// choisit une duree aleatoire comprise dans les bornes 
		SYSparticules[i].vietotale = SYSparticules[i].vie;										// sauvegarde de la vie
	}
}


////////////////////////////////////////////////////
// DefinirTextue definit une texture qui sera utilisee
//	par chacune des particules
////////////////////////////////////////////////////

void Ega_SystemParticule :: DefinirMaterial(Vcb_Material * mat)
{
	SYSmat = mat;				// sauvegarde
}


////////////////////////////////////////////////////
// DefinirTaille definit la taille des particules
////////////////////////////////////////////////////

void Ega_SystemParticule :: DefinirTaille(float taille)
{
	SYStaille = taille;			// sauvegarde
}


////////////////////////////////////////////////////
// DefinirPas definit le pas de viellissement 
//	des particules
////////////////////////////////////////////////////

void Ega_SystemParticule :: DefinirPas(float pas)
{
	SYSpas = pas;				// sauvegarde
}


////////////////////////////////////////////////////
// DefinirAttraction definit l'attraction de chacun 
//	des axes sur les particules
////////////////////////////////////////////////////

void Ega_SystemParticule :: DefinirAttraction(float x,float y,float z)
{
	int i;
	for(i=0;i<SYSnbre;i++)		// pour chacune des particules
	{
		SYSparticules[i].xg = x;		// sauvegarde
		SYSparticules[i].yg = y;
		SYSparticules[i].zg = z;
	}

}


////////////////////////////////////////////////////
// Afficher affiche l'ensemble du systeme et met a 
//	jour les particules "mortes" en les reinitialisant 
//	selon leur type (METEo ou REACTEUR)
//	Prend en compte tous les parametres des systemes :
//	pas de vieillissement, position, vie courane, 
//	vitesse ...
//	S'occupe de faire vieillir les particules et de 
//	les mettre a jour
////////////////////////////////////////////////////

void Ega_SystemParticule :: Afficher()
{
	int i;
	PRINT_PARTICULE2("drawing particule topbegin nbpart = %d\n", SYSnbre);
	//SYSmat->applyMaterial();					// applique la texture correspondante
	glBegin(GL_QUADS);
	for(i=0;i<SYSnbre;i++)						// pour chaque particule
	{
	    PRINT_PARTICULE2("drawing particule between %d\n", SYSparticules[i].active);
		if (SYSparticules[i].active)			// si elle est active
		{
		    PRINT_PARTICULE2("drawing particule between %d\n",SYSparticules[i].x);
		    PRINT_PARTICULE2("                          %d\n",SYSparticules[i].y);
		    PRINT_PARTICULE2("                          %d\n",SYSparticules[i].z);
			float x = SYSparticules[i].x;						
			float y = SYSparticules[i].y;						
			float z = SYSparticules[i].z;					
			//fprintf(stderr, "drawing particule between %d, %d, %d\n", SYSparticules[i].x, SYSparticules[i].y, SYSparticules[i].z);
/*************************** affichage selon type de systeme ********************************/
            PRINT_PARTICULE1("drawing particule begin\n");
			if(SYSreact)						// si c'est un reacteur
			{
				glColor4f(SYSparticules[i].r,SYSparticules[i].v,SYSparticules[i].b,SYSparticules[i].vie/SYSparticules[i].vietotale);		// couleur correspondante

				/* carre de particule */
				glTexCoord2d(0,0); glVertex3f(x-SYStaille/2,y-SYStaille/2,z);
				glTexCoord2d(0,1); glVertex3f(x-SYStaille/2,y+SYStaille/2,z); 
				glTexCoord2d(1,1); glVertex3f(x+SYStaille/2,y+SYStaille/2,z); 
				glTexCoord2d(1,0); glVertex3f(x+SYStaille/2,y-SYStaille/2,z); 
														
				/* mise a jour position */
				SYSparticules[i].x+=SYSvitesse*SYSparticules[i].xd;
				SYSparticules[i].y+=SYSvitesse*SYSparticules[i].yd;
				SYSparticules[i].z+=SYSvitesse*SYSparticules[i].zd;

				/* mise a jour deplacement */
				SYSparticules[i].xd+=SYSparticules[i].xg;			
				SYSparticules[i].yd+=SYSparticules[i].yg;		
				SYSparticules[i].zd+=SYSparticules[i].zg;
				
				/* mise a jour vie */
				SYSparticules[i].vie-=SYSpas;	

				if (SYSparticules[i].vie<0.0)			// si elle est morte				
				{
					float angle = ((float)rand()/RAND_MAX)*360;										// angle aleatoire
					SYSparticules[i].vie = SYSviemin + (((float)rand()/RAND_MAX)*(SYSviemax-SYSviemin));	// vie aleatoire comprise entre les bornes
					SYSparticules[i].vietotale = SYSparticules[i].vie;										// sauvegarde vie totale
					SYSparticules[i].z = SYSx + cos(angle)*(((float)rand()/RAND_MAX)*(SYSreact->diam/2));	// placement aleatoire sur la source				
					SYSparticules[i].y = SYSy + sin(angle)*(((float)rand()/RAND_MAX)*(SYSreact->diam/2));						
					SYSparticules[i].z = SYSz;					
					SYSparticules[i].xd = SYSreact->orx+ (((float)rand()/RAND_MAX)*SYSreact->tolx-(SYSreact->tolx/2));	// generation tolerance aleatoire comprise entre les bornes	
					SYSparticules[i].yd = SYSreact->ory+ (((float)rand()/RAND_MAX)*SYSreact->toly-(SYSreact->toly/2));
					SYSparticules[i].zd = SYSreact->orz+ (((float)rand()/RAND_MAX)*SYSreact->tolz-(SYSreact->tolz/2));	
					if(SYSmor && SYSmor->deg)
					{
						SYSparticules[i].r = SYSmor->rd;
						SYSparticules[i].v = SYSmor->vd;
						SYSparticules[i].b = SYSmor->bd;
					}
				}
			}

			else if(SYSmet)					// si c'est une meteo
			{
				glColor4f(SYSparticules[i].r,SYSparticules[i].v,SYSparticules[i].b,SYSparticules[i].vie/SYSparticules[i].vietotale);		// couleur correspondante

				/* carre de particule */
				glTexCoord2d(0,0); glVertex3f(x-SYStaille/2,y-SYStaille/2,z);
				glTexCoord2d(0,1); glVertex3f(x-SYStaille/2,y+SYStaille/2,z); 
				glTexCoord2d(1,1); glVertex3f(x+SYStaille/2,y+SYStaille/2,z); 
				glTexCoord2d(1,0); glVertex3f(x+SYStaille/2,y-SYStaille/2,z); 

				/* mise a jour position */
				SYSparticules[i].x-=SYSvitesse*SYSparticules[i].xd;
				SYSparticules[i].y-=SYSvitesse*SYSparticules[i].yd;
				SYSparticules[i].z-=SYSvitesse*SYSparticules[i].zd;

				if(SYSmet->mortel)			// si elle est affectee par la vie
				{
					SYSparticules[i].vie-=SYSpas;	// vieillissement
				}

				if (SYSparticules[i].y<=-SYSmet->hauteur/2 || SYSparticules[i].vie<=0.0)		// si elle sort de la zone ou si elle est morte
				{
					SYSparticules[i].x = SYSx+(((float)rand()/RAND_MAX)*SYSmet->longueur) - (SYSmet->longueur/2);	// replacement
					SYSparticules[i].y = SYSmet->hauteur/2;
					SYSparticules[i].z = SYSz+(((float)rand()/RAND_MAX)*SYSmet->largeur) - (SYSmet->largeur/2);
					SYSparticules[i].yd = SYSmet->vitmin + (((float)rand()/RAND_MAX)*(SYSmet->vitmax - SYSmet->vitmin));		// generation vitesse de chute entre les bornes
					SYSparticules[i].vie = SYSviemin + (((float)rand()/RAND_MAX)*(SYSviemax-SYSviemin));					// generation de vie idem			
					SYSparticules[i].vietotale = SYSparticules[i].vie;														// sauvegarde vie totale
					if(SYSmor && SYSmor->deg)
					{
						SYSparticules[i].r = SYSmor->rd;
						SYSparticules[i].v = SYSmor->vd;
						SYSparticules[i].b = SYSmor->bd;
					}

				}
			}

/******** mise a jour selon modele de couleur *************************/

			if(SYSmor)
			{
				float rpas = SYSmor->rdiff/SYSmor->chgvit;
				float vpas = SYSmor->vdiff/SYSmor->chgvit;
				float bpas = SYSmor->bdiff/SYSmor->chgvit;

				if((rpas < 0 && SYSparticules[i].r > SYSmor->ra) || (rpas > 0 && SYSparticules[i].r < SYSmor->ra))
					SYSparticules[i].r += rpas;
				if((vpas < 0 && SYSparticules[i].v > SYSmor->va) || (vpas > 0 && SYSparticules[i].v < SYSmor->va))
					SYSparticules[i].v += vpas;
				if((bpas < 0 && SYSparticules[i].b > SYSmor->ba) || (bpas > 0 && SYSparticules[i].b < SYSmor->ba))
					SYSparticules[i].b += bpas;
			}

			if(SYSarc && !SYSarc->delai)
			{
				SYSparticules[i].r = SYSarc->couleurs[SYSarc->col];
				SYSparticules[i].v = SYSarc->couleurs[SYSarc->col+1];
				SYSparticules[i].b = SYSarc->couleurs[SYSarc->col+2];
				if(SYSarc->col == SYSarc->nbre-1)
					SYSarc->col = 0;
			}
		}
	}
	glEnd();
	/*SYSarc->delai--;
	if(SYSarc->delai == -1)
	{
		SYSarc->col++;
		SYSarc->delai = SYSarc->delaiinit;
	}*/
	//SYSmat->resetMaterial();
}


////////////////////////////////////////////////////
// DefinirNbreParticules definit le nombre de 
//	particules a gerer (utile pour simuler extinction)
//	le sparticules non utilisees ne sont pas effacees
////////////////////////////////////////////////////

void Ega_SystemParticule :: DefinirNbreParticules(int nbre)
{
		SYSnbre = nbre;		// sauvegarde
}


////////////////////////////////////////////////////
// DefinirTaille definit la taille des particules
////////////////////////////////////////////////////

void Ega_SystemParticule :: DefinirVitesse(float vit)
{
	SYSvitesse = vit;		// sauvegarde
}


////////////////////////////////////////////////////
// Replacer redefinit la position de la source
////////////////////////////////////////////////////

void Ega_SystemParticule ::Replacer(float x,float y,float z)
{
	SYSx = x;
	SYSy = y;
	SYSz = z;
}

////////////////////////////////////////////////////
// Destructeur
////////////////////////////////////////////////////

Ega_SystemParticule :: ~Ega_SystemParticule()
{
	free(SYSparticules);		// liberation rssources
}










////////////////////////////////////////////////// FONCTIONS COULEURS ////////////////////////////////////////////////



/**************************************************** ARC_EN_CIEL ******************************************************/



////////////////////////////////////////////////////
// DefinirCouleursArc
////////////////////////////////////////////////////

int Ega_SystemParticule :: DefinirCouleursArc(int nbre,float couleurs[][3],float delai)
{
	if(SYSarc)
	{
		int i;
		SYSarc->couleurs = (float *)couleurs;
		SYSarc->delaiinit = delai;
		SYSarc->delai = SYSarc->delaiinit;
		SYSarc->nbre = nbre;
		SYSarc->col = 0;
		for(i=0;i<SYSnbre;i++)
		{
			SYSparticules[i].r = couleurs[0][1];
			SYSparticules[i].v = couleurs[0][2];
			SYSparticules[i].b = couleurs[0][3];
	
		}	
		return 1;
	}
	else
	{
		PRINT_PARTICULE1("DefinirCouleurArc : ce n'est pas un arc en ciel !\n");
		return 0;
	}
}


/****************************************************** MORPH ******************************************************/



////////////////////////////////////////////////////
// DefinirCouleurDepart
////////////////////////////////////////////////////

int Ega_SystemParticule :: DefinirCouleurDepart(float r,float v,float b)
{
	if(SYSmor)
	{
		int i;
		SYSmor->rd = r;
		SYSmor->vd = v;
		SYSmor->bd = b;
		for(i=0;i<SYSnbre;i++)
		{
			SYSparticules[i].r = SYSmor->rd;
			SYSparticules[i].v = SYSmor->vd;
			SYSparticules[i].b = SYSmor->bd;
		}
		return 1;
	}
	else
	{
		PRINT_PARTICULE1("DefinirCouleurDepart : ce n'est pas un degrade!\n");		// erreur
		return 0;		
	}
}


////////////////////////////////////////////////////
// DefinirCouleurArrivee
////////////////////////////////////////////////////

int Ega_SystemParticule :: DefinirCouleurArrivee(float r,float v,float b)
{
	if(SYSmor)
	{
		SYSmor->ra = r;
		SYSmor->va = v;
		SYSmor->ba = b;
		SYSmor->rdiff = SYSmor->ra - SYSmor->rd;
		SYSmor->vdiff = SYSmor->va - SYSmor->vd;
		SYSmor->bdiff = SYSmor->ba - SYSmor->bd;
		return 1;
	}
	else
	{
		PRINT_PARTICULE1("DefinirCouleurDepart : ce n'est pas un degrade!\n");		// erreur
		return 0;		
	}
}


////////////////////////////////////////////////////
// InitMorph
////////////////////////////////////////////////////

int Ega_SystemParticule :: InitMorph(float vit,bool deg)
{
	if(SYSmor)
	{
		SYSmor->chgvit = vit;
		SYSmor->deg = deg;
		return 1;
	}
	else
	{
		PRINT_PARTICULE1("DefinirCouleurDepart : ce n'est pas un degrade!\n");		// erreur
		return 0;		
	}
}

/****************************************************** UNIQUE ******************************************************/



////////////////////////////////////////////////////
// DefinirCouleurUnique definit une couleur unique
//	pour toutes les particules r,v,b
////////////////////////////////////////////////////

void Ega_SystemParticule :: DefinirCouleurUnique(float r,float v,float b)
{
	int i;
	for(i=0;i<SYSnbre;i++)			// pour chaque particule
	{
		SYSparticules[i].r = r;		// sauvegarde couleur (rouge)
		SYSparticules[i].v = v;		// (vert)
		SYSparticules[i].b = b;		// (bleu)
	}
}



/****************************************************** ALEATOIRE ******************************************************/



////////////////////////////////////////////////////
// DefinirCouleurs
////////////////////////////////////////////////////

void Ega_SystemParticule :: DefinirCouleurs(int nbre,float couleurs[][3])
{
	int i;
	int id;
	nbre--;
	srand(time(NULL));		// init random
	if(couleurs)
	{
		for(i=0;i<SYSnbre;i++)
		{
			id = (int)(i*((float)nbre/SYSnbre));
			SYSparticules[i].r = couleurs[id][0];
			SYSparticules[i].v = couleurs[id][1];
			SYSparticules[i].b = couleurs[id][2];
		}
	}
	else
	{
		for(i=0;i<SYSnbre;i++)
		{
			SYSparticules[i].r = ((float)rand()/RAND_MAX);
			SYSparticules[i].v = ((float)rand()/RAND_MAX);
			SYSparticules[i].b = ((float)rand()/RAND_MAX);
		}	
	}
}









///////////////////////////////////////////////   FONCTIONS TYPE DE SYSTEME   /////////////////////////////////////////////////



/****************************************************   type METEO   ********************************************************/


	
////////////////////////////////////////////////////
// InitZoneMeteo
////////////////////////////////////////////////////

int Ega_SystemParticule :: InitZoneMeteo(float x,float y,float z,float longueur,float hauteur,float largeur)
{
	if(SYSmet)								// s'il s'agit d'une meteo
	{
		int i;
		SYSmet->hauteur = hauteur;		// sauvegardes
		SYSmet->largeur = largeur;
		SYSmet->longueur = longueur;
		SYSx = x;
		SYSy = y;
		SYSz = z;
		srand(time(NULL));				// init random
		for(i=0;i<SYSnbre;i++)			// pour chaque particule
		{
			SYSparticules[i].x = SYSx+(((float)rand()/RAND_MAX)*SYSmet->longueur) - (SYSmet->longueur/2);	// placement en x a l'interieur de la zone
			SYSparticules[i].y = SYSmet->hauteur/2;							// placement en y
			SYSparticules[i].z = SYSz+(((float)rand()/RAND_MAX)*SYSmet->largeur) - (SYSmet->largeur/2);	// placement en z
		}
		return 1;			// OK
	}
	else									// sinon
	{
		PRINT_PARTICULE1("InitZoneMeteo : ce systeme n'est pas une meteo !\n");		// erreur
		return 0;			// pas OK
	}	
}


////////////////////////////////////////////////////
// InitZoneMeteo
////////////////////////////////////////////////////

int Ega_SystemParticule :: InitMeteo(float ymax,float ymin,bool mort)
{
	if(SYSmet)								// s'il s'agit d'une meteo
	{
		int i;
		SYSmet->mortel = mort;		// sauvegardes
		SYSmet->vitmin = ymin;
		SYSmet->vitmax = ymax;

		srand(time(NULL));			// init random
		for(i=0;i<SYSnbre;i++)		// pour chaque particule
		{	
			SYSparticules[i].xd = 0.0;		// deplacement en x
			SYSparticules[i].yd = SYSmet->vitmin + (((float)rand()/RAND_MAX)*(SYSmet->vitmax-SYSmet->vitmin));		// deplacement en y aleatoire (=vitese de chute)
			SYSparticules[i].zd = 0.0;		// deplacement en z
		}
		return 1;
	}
	else									// sinon
	{
		PRINT_PARTICULE1("InitMeteo : ce systeme n'est pas une meteo !\n");		// erreur
		return 0;			// pas OK

	}

}




/****************************************************   type REACTEUR   ********************************************************/



////////////////////////////////////////////////////
// InitReacteur initialise la position et le diametre
//	de la source du reacteur
//	1 si OK 0 sinon
////////////////////////////////////////////////////

int Ega_SystemParticule :: InitReacteur(float x,float y,float z,float diam)
{
	if(SYSreact)							// s'il s'agit bien d'un reacteur
	{
		int i;
		float angle;
		SYSreact->diam = diam;				// diametre propre au reacteur
		SYSx = x;							// sauvegarde des coordonnees source
		SYSy = y;
		SYSz = z;
		srand(time(NULL));					// initialise le random
		for(i=0;i<SYSnbre;i++)				// pour chaque particule
		{
			angle = ((float)rand()/RAND_MAX)*360;		// definition d'un angle aleatoire
			SYSparticules[i].x = SYSx+cos(angle)*(((float)rand()/RAND_MAX)*(SYSreact->diam/2));	// placement en x a l'interieur du cercle source
			SYSparticules[i].y = SYSy+sin(angle)*(((float)rand()/RAND_MAX)*(SYSreact->diam/2));	// placement en y a l'interieur du cercle source
			SYSparticules[i].z = SYSz;															// placement en z
		}
		return 1;			// OK
	}
	else									// sinon
	{
		PRINT_PARTICULE1("InitReacteur : ce systeme n'est pas un reacteur !\n");		// erreur
		return 0;			// pas OK
	}
}

////////////////////////////////////////////////////
// InitReacteur initialise la position et le diametre
//	de la source du reacteur
//	1 si OK 0 sinon
////////////////////////////////////////////////////

int Ega_SystemParticule :: replaceReacteur(Vect3 pos)
{
	if(SYSreact)							// s'il s'agit bien d'un reacteur
	{
		SYSx = pos[0];							// sauvegarde des coordonnees source
		SYSy = pos[1];
		SYSz = pos[2];
	}
	else									// sinon
	{
		PRINT_PARTICULE1("InitReacteur : ce systeme n'est pas un reacteur !\n");		// erreur
		return 0;			// pas OK
	}
	return 1;
}
////////////////////////////////////////////////////
// OrienterReacteur permet d'orienter le reacteur 
//	sur un axe x,y,z et definit "l'explosion" initiale
//	avec xcoeff,ycoeff,zcoeff (+petit, +petite sera explo)
//	<0.5
//	1 si OK 0 sinon
////////////////////////////////////////////////////

int Ega_SystemParticule :: OrienterReacteur(float x,float y,float z, float xcoeff, float ycoeff, float zcoeff)
{
	if(SYSreact)						// s'il s'agit bien d'un reacteur
	{
		int i;
		SYSreact->orx = x;				// sauvegarde de l'orientation
		SYSreact->ory = y;
		SYSreact->orz = z;
		srand(time(NULL));				// init random
		for(i=0;i<SYSnbre;i++)			// pour chaque particule
		{
			SYSparticules[i].xd = SYSreact->orx + float((rand()%50)-26.0)*xcoeff;	// deplacement des particules selon orientation + coeff d'explosion
			SYSparticules[i].yd = SYSreact->ory + float((rand()%50)-26.0)*ycoeff;
			SYSparticules[i].zd = SYSreact->orz + float((rand()%50)-26.0)*zcoeff;
		}
		return 1;			// OK
	}
	else								// sinon
	{
		PRINT_PARTICULE1("OrienterReacteur : ce systeme n'est pas un reacteur !\n");		// erreur	
		return 0;			// pas OK
	}

}


////////////////////////////////////////////////////
// DefinirToleranceReacteur definit la facon dont
//	les particules vont se repandre sur les axes
//	x,y,z
//	1 si OK 0 sinon
////////////////////////////////////////////////////

int Ega_SystemParticule :: DefinirToleranceReacteur(float x,float y,float z)
{
	if(SYSreact)					// s'il s'agit bien d'un reacteur
	{
		int i;
		SYSreact->tolx = x;			// sauvegarde des tolerances
		SYSreact->toly = y;
		SYSreact->tolz = z;
		srand(time(NULL));			// init random
		for(i=0;i<SYSnbre;i++)		// pour chaque particule
		{
			SYSparticules[i].xd +=  (((float)rand()/RAND_MAX)*SYSreact->tolx-(SYSreact->tolx/2));	// deplacement des particules tient compte des tolerances aleatoires
			SYSparticules[i].yd +=  (((float)rand()/RAND_MAX)*SYSreact->toly-(SYSreact->toly/2));
			SYSparticules[i].zd +=  (((float)rand()/RAND_MAX)*SYSreact->tolz-(SYSreact->tolz/2));
		}
		return 1;			// OK
	}	
	else							// sinon
	{
		PRINT_PARTICULE1("DefinirToleranceReacteur : ce systeme n'est pas un reacteur !\n");		// erreur
		return 0;			// pas OK
	}
}


////////////////////////////////////////////////////
// ReOrienterReacteur reoriente le reacteur vers
//	x,y,z
////////////////////////////////////////////////////

int Ega_SystemParticule :: ReOrienterReacteur(float x,float y,float z)
{
	if(SYSreact)
	{
		SYSreact->orx = x;
		SYSreact->ory = y;
		SYSreact->orz = z;
		return 1;
	}
	else
	{
		PRINT_PARTICULE1("ReOrienterReacteur : ce systeme n'est pas un reacteur !\n");		// erreur
		return 0;
	}
}
