/*
 * Copyright (C) 2005  ENSI-GAME-ART
 *
 * Club ENSI-GAME-ART ENSICAEN
 * 6 Boulevard Marechal Juin
 * F-14050 Caen Cedex
 *
 * Ce fichier est l'oeuvre d'eleves de l'ENSI de Caen. Il est developp
 * sous la licence GPL :
 *
 ****************************************************************
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *****************************************************************
 *
 * Ce fichier fait partie du projet de jeu video de course spatiale
 * developpe au sein du club ENSI-GAME-ART
 */
 
 #ifndef SYSTPARTICULE
#define SYSTPARTICULE

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <gl/gl.h>
#include "display/Vcb_Material.h"

// define pour type de systeme de particule 

#define METEO		0x00000002
#define REACTEUR	0x00000003

// define pour generation des couleurs

#define	UNIQUE		0x00000010
#define ALEATOIRE	0x00000020
#define DEGRADE		0x00000030
#define ARC_EN_CIEL 0x00000040
#define MORPH		0x00000050


struct Particule{
	bool	active;					// si oui ou non la particule peut etre affichee
	float	vie;					// "vie" courante de la particule
	float	vietotale;				// "vie" totale de la particule
	float	r;						// rouge
	float	v;						// vert
	float	b;						// bleu
	float	x;						// position en x
	float	y;						// position en y
	float	z;						// position en z
	float	xd;						// direction en x
	float	yd;						// direction en y
	float	zd;						// direction en z
	float	xg;						// gravite en x
	float	yg;						// gravite en y
	float	zg;						// gravite en z	
};

/* structures de type de systeme */

struct Reacteur{
	float diam;						// diametre propre au reacteur
	float orx;						// orientation en x
	float ory;						// orientation en y
	float orz;						// orientation en z
	float tolx;						// tolerance en x (eparpillement des particule sur cet axe)
	float toly;						// tolerance en y (eparpillement des particule sur cet axe)
	float tolz;						// tolerance en z (eparpillement des particule sur cet axe)
};

struct Meteo{
	float longueur;					// longueur de la zone affectee
	float largeur;					// largeur de la zone affectee
	float hauteur;					// hauteur de la zone affectee
	float vitmin;					// vitesse minimum de chute
	float vitmax;					// vitesse maximum de chute
	bool mortel;					// affectee ou non par la vie ?

};

/* structures pour modele de couleurs */

struct Morph{
	float rd;
	float vd;
	float bd;
	float ra;
	float va;
	float ba;
	float rdiff;
	float vdiff;
	float bdiff;
	float chgvit;
	bool deg;
};

struct ArcEnCiel{
	float delai;
	float delaiinit;
	float *couleurs;
	int nbre;
	int col;
};


/**
 * Manage a set of particules.
 *
 * @author Pierre Leduc
 */
class Ega_SystemParticule
{
private:

	Particule * SYSparticules;		// tableau de particules
	Vcb_Material * SYSmat;				// texture associee
	int SYStype;					// type de systeme de particules
	int SYSnbre;					// nombre de particules a gerer
	float SYStaille;				// taille des particules
	float SYSpas;					// pas de viellissement
	float SYSviemin;				// vie minimale d'une particule
	float SYSviemax;				// vie maximale d'une particule
	float SYSx;						// position en x d'un repere du systeme
	float SYSy;						// idem en y
	float SYSz;						// idem en z
	float SYSvitesse;				// vitesse des particules
	Reacteur * SYSreact;			// structure reacteur
	Meteo * SYSmet;					// structure meteo
	Morph * SYSmor;					// structure morph
	ArcEnCiel * SYSarc;					// structure arc

public:

/* FONCTIONS GENERALES */

	Ega_SystemParticule(int,int);					// constructeur (nbre de particules,type de systeme)
	void DefinirDureeDeVie(float,float);		// pour specifier la duree de vie (max,min)
	void DefinirMaterial(Vcb_Material * mat);			// pour specifier une texture a utiliser
	void DefinirTaille(float);					// pour specifier la taille
	void DefinirPas(float);						// pour specifier le pas de vieillissement
	void DefinirAttraction(float,float,float);	// specifie l'attraction (x,y,z);
	void DefinirNbreParticules(int);			// specifie le nbre de particules
	void DefinirVitesse(float);					// specifie la vitesse de deplacement des particules
	void Replacer(float,float,float);
	void Afficher();							// affichage
	virtual ~Ega_SystemParticule();						// destructeur

/* FONCTIONS COULEURS */

	/* pour MORPH */
	int InitMorph(float,bool);								// initialise le morph (vitesse de changement, degrade ou pas)
	int DefinirCouleurDepart(float,float,float);			// couleur de depart du degrade (rouge,vert,bleu)
	int DefinirCouleurArrivee(float,float,float);			// couleur finale (rouge,vert,bleu)

	/* pour UNIQUE */
	void DefinirCouleurUnique(float,float,float);			// couleur unique (rouge,vert,bleu)

	/* pour ALEATOIRE */
	void DefinirCouleurs(int,float [][3]);					// (nbre de couleurs, tableau de couleurs), si tableau de couleurs = NULL, generation aleatoire

	/* pour ARC_EN_CIEL */
	int DefinirCouleursArc(int nbre,float couleurs[][3],float delai);	// (nombre de couleurs, tableau de couleurs, delai avant changement de couleur)

/* FONCTIONS DE TYPE */

	/* pour type METEO */
	int InitMeteo(float,float,bool);							// initialise la vitesse de chute des particules et precise si elle sont "mortelles"
	int InitZoneMeteo(float,float,float,float,float,float);		// zone affectee (x,y,z,longueur,hauteur,largeur)

	/* pour type REACTEUR */
	int InitReacteur(float,float,float,float);					// source reacteur (x,y,z,diametre source)
	int replaceReacteur(Vect3 pos);

	int OrienterReacteur(float,float,float,float,float,float);	// pour orienter sur un axe le reacteur et specifier le demarrage (x,y,z,xcoeff,ycoeff,zcoeff)
	int DefinirToleranceReacteur(float,float,float);			// pour definir comment vont se repandre les particules (tolerancex, tolerancey, tolerancez)
	int ReOrienterReacteur(float,float,float);					// pour reorienter le reacteur vers un autre axe (x,y,z)
};

#endif

