/*
 * Copyright (C) 2005  ENSI-GAME-ART
 *
 * Club ENSI-GAME-ART ENSICAEN
 * 6 Boulevard Marechal Juin
 * F-14050 Caen Cedex
 *
 * Ce fichier est l'oeuvre d'eleves de l'ENSI de Caen. Il est developp
 * sous la licence GPL :
 *
 ****************************************************************
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *****************************************************************
 *
 * Ce fichier fait partie du projet de jeu video de course spatiale
 * developpe au sein du club ENSI-GAME-ART
 */
 
#include "display/Ega_Texture.h"
#include "other/Ega_debug.h"


Ega_Texture::Ega_Texture()
{
    tex_isTransparent = false;
	tex_cree = false;
}

Ega_Texture::Ega_Texture( char *Filename )
{
    tex_isTransparent = false;
	tex_cree = false;
	loadTexture( Filename, "" );	
}

Ega_Texture::Ega_Texture( char *Filename, char * Maskname )
{
    tex_isTransparent = false;
	tex_cree = false;
	loadTexture( Filename, Maskname );	
}

Ega_Texture::~Ega_Texture()
{
	if(tex_cree)
	{
		free(tex_pixels);
	}
	glDeleteTextures(1,&tex_id);
}

GLuint Ega_Texture::loadTexture( char *Filename )
{
    return loadTexture( Filename, "" );
}

GLuint Ega_Texture::loadTexture( char *Filename, char *Maskname )								
{

	//if(tex_pixels != NULL)
	//	free(tex_pixels);
		
   
    if( tex_image = IMG_Load(Filename) )
    {

        tex_filename = Filename;
        tex_depth = tex_image->format->BitsPerPixel;
        tex_width = tex_image->w;
        tex_height = tex_image->h;
   
        tex_pixels = (unsigned char *)tex_image->pixels; 
        
        if(!tex_cree)
		  glGenTextures(1, &tex_id);					
	
		glBindTexture(GL_TEXTURE_2D, tex_id);  
  		
  	
  		
        if( tex_mask = IMG_Load(Maskname) )
        {
            unsigned char *result = new unsigned char[tex_width*tex_height*4];
            unsigned char *pix = tex_pixels;
            unsigned char *pixM = (unsigned char *)tex_mask->pixels;
			// pour lire  l'envers ( de haut en bas)
            pix += (tex_height-1)*tex_width*3;
            pixM += (tex_height-1)*tex_width*3;
            
            for(int i=0; i<tex_height; i++)
            {
                for(int j=0; j<tex_width; j++)
                {
                     result[i*tex_width*4+j*4] = *pix; pix+=1;
                     result[i*tex_width*4+j*4+1] = *pix; pix+=1;
                     result[i*tex_width*4+j*4+2] = *pix; pix+=1;
                     result[i*tex_width*4+j*4+3] = (*pixM + *(pixM+1) + *(pixM+2)) / 3; pixM+=3;
                }
				// on remonte de 2 lignes (la fin de la ligne n => le debut de la ligne n-1)
                pix -= 2*tex_width*3;
                pixM -= 2*tex_width*3;
            }  
            glTexImage2D(GL_TEXTURE_2D,0,  GL_RGBA, tex_width, tex_height, 0, GL_RGBA, GL_UNSIGNED_BYTE, result);
            delete[] result;
            
            tex_isTransparent = true;
            
            SDL_FreeSurface( tex_mask );
        }        
        else
        {
            unsigned char *result = new unsigned char[tex_width*tex_height*3];
            unsigned char *pix = tex_pixels;
            pix += (tex_height-1)*tex_width*3;
            
            for(int i=0; i<tex_height; i++)
            {
                for(int j=0; j<tex_width; j++)
                {
                     result[i*tex_width*3+j*3] = *pix; pix+=1;
                     result[i*tex_width*3+j*3+1] = *pix; pix+=1;
                     result[i*tex_width*3+j*3+2] = *pix; pix+=1;
                }    
                pix -= 2*tex_width*3;
            }  
            glTexImage2D(GL_TEXTURE_2D,0,  GL_RGB, tex_width, tex_height, 0, GL_RGB, GL_UNSIGNED_BYTE,  result);
            delete[] result;
            
            tex_isTransparent = false;
        }   

        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
        
        tex_cree = true;
	}
	else
    {
        tex_cree = false;
        return 0;
    }    
	SDL_FreeSurface( tex_image );
	resetTexture();

	return tex_id;										// Return The Status
}

void Ega_Texture::applyTexture()
{
	if(tex_cree){
        glEnable(GL_TEXTURE_2D);
	    glBindTexture(GL_TEXTURE_2D,tex_id);	  
	}        
    PRINT_DISPLAY2("tex_id applytex: %d\n", tex_id);  
  
}

void Ega_Texture::resetTexture()
{
    
    if(tex_cree)
        glBindTexture(GL_TEXTURE_2D, 0 );
    //glDisable(GL_TEXTURE_2D);    
    PRINT_DISPLAY2("tex_id resettex: %d\n", tex_id); 
        
}    
GLuint Ega_Texture::getID()
{
    return tex_id;
}

int Ega_Texture::getWidth()
{
    return tex_width;
}

int Ega_Texture::getHeight()
{
    return tex_height;
}

int Ega_Texture::getDepth()
{
    return tex_depth;
}

bool Ega_Texture::isTransparent()
{
    return tex_isTransparent;
}



