 /*
 * Copyright (C) 2005  ENSI-GAME-ART
 *
 * Club ENSI-GAME-ART ENSICAEN
 * 6 Boulevard Marechal Juin
 * F-14050 Caen Cedex
 *
 * Ce fichier est l'oeuvre d'eleves de l'ENSI de Caen. Il est developp
 * sous la licence GPL :
 *
 ****************************************************************
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *****************************************************************
 *
 * Ce fichier fait partie du projet de jeu video de course spatiale
 * developpe au sein du club ENSI-GAME-ART
 */
 

#include "engine/Ega_Scene.h"
#include "display/Vcb_Light.h"
#include "other/Ega_debug.h"

Vcb_Light::Vcb_Light()
{    
    initParams();
}

Vcb_Light::~Vcb_Light()
{
    
}

void Vcb_Light::store()
{
    Ega_Scene::getIt()->addLight(this);
}

void Vcb_Light::initParams()
{
    lig_ambientColor = Vect4( 0.0, 0.0, 0.0, 1.0 );
    lig_diffColor = Vect4( 0.0, 0.0, 0.0, 1.0 );
    lig_specColor = Vect4( 0.0, 0.0, 0.0, 1.0 );
    lig_pos = Vect4( 0.0, 0.0, 1.0, 0.0 );
    lig_spotDirection = Vect3( 0.0, 0.0, -1.0 );
    lig_spotAngle = 180.0;
    lig_spotConcentrate = 0.0;
}

void Vcb_Light::setLight()
{    
    glLightfv( lig_id, GL_AMBIENT, vect4TOfloat(lig_ambientColor, tmpColor) );
    glLightfv( lig_id, GL_DIFFUSE, vect4TOfloat(lig_diffColor, tmpColor) );
    glLightfv( lig_id, GL_SPECULAR, vect4TOfloat(lig_specColor, tmpColor) );
    glLightfv( lig_id, GL_POSITION, vect4TOfloat(lig_pos, tmpColor) );
    glLightfv( lig_id, GL_SPOT_DIRECTION, vect3TOfloat(lig_spotDirection, tmpDirection) );  
    glLightfv( lig_id, GL_SPOT_CUTOFF, &lig_spotAngle ); 
    glLightfv( lig_id, GL_SPOT_EXPONENT, &lig_spotConcentrate );
}  

void Vcb_Light::setID( GLenum id )
{
    lig_id = id;
    setLight();
}

void Vcb_Light::setAmbientColor(Vect4 _ambi)   
{ 
    lig_ambientColor = _ambi;
    glLightfv( lig_id, GL_AMBIENT, vect4TOfloat(lig_ambientColor, tmpColor) );
}

void Vcb_Light::setDiffColor(Vect4 _diff)      
{ 
    lig_diffColor = _diff;
    glLightfv( lig_id, GL_DIFFUSE, vect4TOfloat(lig_diffColor, tmpColor) );
}

void Vcb_Light::setSpecColor(Vect4 _spec)      
{ 
    lig_specColor = _spec;
    glLightfv( lig_id, GL_SPECULAR, vect4TOfloat(lig_specColor, tmpColor) );
}

void Vcb_Light::setPos(Vect4 _pos)             
{ 
    lig_pos = _pos;
    glLightfv( lig_id, GL_POSITION, vect4TOfloat(lig_pos, tmpColor) );
}

void Vcb_Light::setSpotDirection(Vect3 _dir)   
{ 
    lig_spotDirection = _dir;
    glLightfv( lig_id, GL_SPOT_DIRECTION, vect3TOfloat(lig_spotDirection, tmpDirection) );   
}

void Vcb_Light::setSpotAngle(GLfloat _ang)         
{ 
    lig_spotAngle = _ang;
    glLightfv( lig_id, GL_SPOT_CUTOFF, &lig_spotAngle );  
}

void Vcb_Light::setSpotConcentrate(GLfloat _conc)  
{ 
    lig_spotConcentrate = _conc;
    glLightfv( lig_id, GL_SPOT_EXPONENT, &lig_spotConcentrate ); 
}


void Vcb_Light::applyLight()
{
    if( !glIsEnabled(GL_LIGHTING) )
        glEnable(GL_LIGHTING);
        
    PRINT_DISPLAY2("apply light %d",  lig_id);   
   /* glEnable(lig_id);
    glBegin(GL_POINT);
    glPointSize(10.0);
    glColor3ub(255,255,255);
    glVertex3f(lig_pos[0], lig_pos[1], lig_pos[2]);
    glEnd();*/
}

void Vcb_Light::resetLight()
{
    glDisable(lig_id);
    glDisable(GL_LIGHTING);
}
