/*
 * Copyright (C) 2005  ENSI-GAME-ART
 *
 * Club ENSI-GAME-ART ENSICAEN
 * 6 Boulevard Marechal Juin
 * F-14050 Caen Cedex
 *
 * Ce fichier est l'oeuvre d'eleves de l'ENSI de Caen. Il est developp
 * sous la licence GPL :
 *
 ****************************************************************
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *****************************************************************
 *
 * Ce fichier fait partie du projet de jeu video de course spatiale
 * developpe au sein du club ENSI-GAME-ART
 */

#ifndef _VCB_LIGHT_H_
#define _VCB_LIGHT_H_

#include "tool/Ega_Tools.h"
#include "display/Ega_Texture.h"
#include <gl/gl.h>
/**
 * Class for light.
 * 
 *
 * @author Pierre Leduc
 * @author Matthieu Gautier
 * @version	1.0
 */
class Vcb_Light  
{
public:
	
	/*
	 * Create a basic light.
	 */
    Vcb_Light( );
	
	/*
	 * "Unstore" the light before deleting
	 */
	virtual ~Vcb_Light();
	
private:
    
   	GLenum   lig_id;
	Vect4    lig_ambientColor;
	Vect4    lig_diffColor;
	Vect4    lig_specColor;
	Vect4    lig_pos;
	Vect3    lig_spotDirection;
	GLfloat  lig_spotAngle;
	GLfloat  lig_spotConcentrate;
	
	float tmpColor[4]; //for Vect to color conversion
	float tmpDirection[3];
	
public:
    
    /*
     * Store the light on the Lightmanager
	 */
    void store();
    
    /*
     * Record himself on OpenGl
     * @param _id identifier of the light for OpenGl
     */
	void setID(GLenum _id);	
	
	/*
	 * Spcifier la couleur ambient
	 * @param _ambi couleur ambient
	 */
	void setAmbientColor(Vect4 _ambi);
	
	/*
	 * Spcifier la couleur diffuse
	 * @param _diff couleur diffuse
	 */
	void setDiffColor(Vect4 _diff);
	
	/*
	 * Spcifier la couleur spculaire
	 * @param _spec couleur spculaire
	 */
	void setSpecColor(Vect4 _spec);
	
	/*
	 * Spcifier la position de la lumiere
	 * @param _pos position de la lumiere
	 */
	void setPos(Vect4 _pos);
	
	/*
	 * Spcifier la direction de la lumiere
	 * @param _dir direction de la lumiere
	 */
	void setSpotDirection(Vect3 _dir);
	
	/*
	 * Spcifier l'angle du cone de lumiere
	 * @param _ang angle du cone de lumiere
	 */
	void setSpotAngle(GLfloat _ang);
	
	/*
	 * Spcifier la "dispertion" de la lumiere
	 * @param _conc dispertion de la lumiere
	 */
	void setSpotConcentrate(GLfloat _conc);
	
	/*
	 * initialise la lumiere avec les params de base
	 */
	void initParams();
	
	/*
     * S'enregistre auprs d'openGl
     */
	void setLight();

	/*
	 * s'active ds openGl
	 */
	void applyLight();
	
	/*
	 * dsactive toutes les lumiere
	 */
	void resetLight();

};

#endif
