 /*
 * Copyright (C) 2005  ENSI-GAME-ART
 *
 * Club ENSI-GAME-ART ENSICAEN
 * 6 Boulevard Marechal Juin
 * F-14050 Caen Cedex
 *
 * Ce fichier est l'oeuvre d'eleves de l'ENSI de Caen. Il est developp
 * sous la licence GPL :
 *
 ****************************************************************
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *****************************************************************
 *
 * Ce fichier fait partie du projet de jeu video de course spatiale
 * developpe au sein du club ENSI-GAME-ART
 */

#include "engine/Ega_Scene.h"
#include "display/Vcb_Material.h"
#include "other/Ega_debug.h"


Vcb_Material::Vcb_Material()
{    
    mat_hasDiffTex = false;mat_diffTex = NULL;
    mat_hasBumpTex = false;mat_bumpTex = NULL;
    
    //mat_id = -1;
    mat_isShared = false;
    mat_shininess = 0.0;
    mat_opacity = 1.0;
    mat_diffColor = Vect4(1.0, 1.0, 1.0,1.0);
    mat_ambientColor = Vect4(1.0, 1.0, 1.0, 1.0);
    mat_specColor = Vect4(1.0, 1.0, 1.0, 1.0);
    mat_emisColor = Vect4(0.0, 0.0, 0.0, 0.0);
}    


Vcb_Material::Vcb_Material( char * fileName, char * bumpName )
{
    //mat_id = -1;
    mat_isShared = false;
    mat_shininess = 0.0;
    mat_opacity = 1.0;
    mat_diffColor = Vect4(1.0, 1.0, 1.0,1.0);
    mat_ambientColor = Vect4(1.0, 1.0, 1.0, 1.0);
    mat_specColor = Vect4(1.0, 1.0, 1.0, 1.0);
    mat_emisColor = Vect4(0.0, 0.0, 0.0, 0.0);
    
    if(fileName) {
    mat_diffTex = new Ega_Texture( fileName );  mat_hasDiffTex = true;  }
    else if(bumpName) {
    mat_bumpTex = new Ega_Texture( bumpName );  mat_hasBumpTex = true;  }    
}

Vcb_Material::~Vcb_Material()
{
    
}

GLuint Vcb_Material::store()
{
    PRINT_DISPLAY1("Store material");
    return Ega_Scene::getIt()->addMaterial(this);
} 

void Vcb_Material::setDiffTexture( char * fileName, char *maskName )
{
	PRINT_LOADER2("chargement texture : %s\n", fileName);
	PRINT_LOADER2("chargement mask : %s\n", maskName);
    mat_diffTex = new Ega_Texture();
    if( mat_diffTex->loadTexture( fileName , maskName ) )
    {  mat_hasDiffTex = true;    }
}

void Vcb_Material::setDiffTexture( char * fileName )
{
	PRINT_LOADER2("chargement texture : %s\n", fileName);
    mat_diffTex = new Ega_Texture();
    if( mat_diffTex->loadTexture( fileName) )
    {  mat_hasDiffTex = true;    }
}

void Vcb_Material::setBumpTexture( char * fileName )
{
    mat_bumpTex = new Ega_Texture();
    if( mat_bumpTex->loadTexture( fileName ) )
    {  mat_hasBumpTex = true;  }      
}

void Vcb_Material::applyMaterial()
{
    if(glIsEnabled(GL_LIGHTING))
    {
        glPushAttrib( GL_TEXTURE_BIT );
        glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
    }
    else
    {
        glPushAttrib( GL_TEXTURE_BIT );
        glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
    }        
    

    glColor4f( 1.0, 1.0, 1.0, mat_opacity );  
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    
    if(mat_hasDiffTex){
    if ( mat_diffTex->isTransparent() )
    {
       //glEnable(GL_CULL_FACE);
    }
    }        
        
    glMaterialf( GL_FRONT_AND_BACK, GL_SHININESS, mat_shininess);
    glMaterialfv( GL_FRONT_AND_BACK, GL_DIFFUSE, vect4TOfloat(mat_diffColor, tmpColor));  
    glMaterialfv( GL_FRONT_AND_BACK, GL_AMBIENT, vect4TOfloat(mat_ambientColor, tmpColor));
    glMaterialfv( GL_FRONT_AND_BACK, GL_SPECULAR, vect4TOfloat(mat_specColor, tmpColor));
    glMaterialfv( GL_FRONT_AND_BACK, GL_EMISSION, vect4TOfloat(mat_emisColor, tmpColor)); 
    
    if(mat_hasDiffTex)
        mat_diffTex->applyTexture();   
}

void Vcb_Material::resetMaterial()
{
    if(mat_hasDiffTex){mat_diffTex->resetTexture();
    if ( mat_diffTex->isTransparent() )
    {
        glColor4f( 1.0, 1.0, 1.0, 1.0 );
       // glDisable(GL_CULL_FACE);
    } 
    }       
    
    glDisable(GL_BLEND);
    //glAlphaFunc(GL_ALWAYS, 0.0);
    glPopAttrib();   
}


bool Vcb_Material::isShared()
{
    return mat_isShared;
}

bool Vcb_Material::hasDiffTex()
{
    return mat_hasDiffTex;
}
bool Vcb_Material::hasBumpTex()
{
    return mat_hasBumpTex;
}
