 /*
 * Copyright (C) 2005  ENSI-GAME-ART
 *
 * Club ENSI-GAME-ART ENSICAEN
 * 6 Boulevard Marechal Juin
 * F-14050 Caen Cedex
 *
 * Ce fichier est l'oeuvre d'eleves de l'ENSI de Caen. Il est developp
 * sous la licence GPL :
 *
 ****************************************************************
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *****************************************************************
 *
 * Ce fichier fait partie du projet de jeu video de course spatiale
 * developpe au sein du club ENSI-GAME-ART
 */
 
#ifndef _EGA_MATERIAL_H_
#define _EGA_MATERIAL_H_

#include "tool/Ega_Tools.h"
#include "display/Ega_Texture.h"
#include <gl/gl.h>

/*
 * Class for material.
 * Use Vcb_Texture
 *
 * @author Pierre Leduc
 * @author Matthieu Gautier
 * @version	1.0
 * @see Vcb_Texture
 */
class Vcb_Material  
{
public:
    /*
     * Crer un material de base sans texture
     */
	Vcb_Material();
	
	/*
	 * Crer un matrial de base avec les texture diffuse et bump spcifi
	 * @param fileName nom du fichier texture diffuse
	 * @param bumpName non du fichier texture de bump
	 */
	 
	Vcb_Material( char * fileName, char * bumpName );
	
	/*
	 * destructeur
	 */
	virtual ~Vcb_Material();
	
private:
    
   	GLuint   mat_id;
   	bool     mat_isShared;
	double   mat_shininess;
	double   mat_opacity;
	Vect4    mat_ambientColor;
	Vect4    mat_diffColor;
	Vect4    mat_specColor;
	Vect4    mat_emisColor;
	
	bool mat_hasDiffTex;
	bool mat_hasBumpTex;
 			    			
	Ega_Texture *mat_diffTex;
	Ega_Texture *mat_bumpTex;
	
	float tmpColor[4];      //for Vect to color conversion
	
public:
    
    /*
     * S'enregistre auprs du manager de matriaux
     * @return id of the material
     */
    GLuint store();
    
    /*
     * specifie the diffuse texture with a mask
     * @param fileName name of the texture file
     * @param maskName name of the mask texture file
     */
    void setDiffTexture( char * fileName, char * maskName );
    
    /*
     * specifie the diffuse texture without mask
     * @param fileName name of the texture file
     */
    void setDiffTexture( char * fileName );
    
    /*
     * specifie the Bump texture file
     * @param fileName name of the bump texture file
     */
    void setBumpTexture( char * fileName );
    
    /*
     * set the id of the material
     * @param _id identifier of the material
     */
	GLuint setID(GLuint _id)               
    { mat_id = _id;};
    
    /*
     * set the Shininess of the material
     * @param _shin shininess value to set
     */
	void setShininess(double _shin)        
    { mat_shininess = _shin;};
    
    /*
     * set the opacity of the material
     * @param _opac opacity value to set
     */
	void setOpacity(double _opac)          
    { mat_opacity = _opac;};
    
    /*
     * set the ambient color of the material
     * @param _ambi ambient color to set
     */
	void setAmbientColor(Vect4 _ambi)   
    { mat_ambientColor = _ambi;};
    
    /*
     * set the  diffuse color of the material
     * @param _diff diffuse color to set
     */
	void setDiffColor(Vect4 _diff)      
    { mat_diffColor = _diff;};
    
    /*
     * set the specular color of the material
     * @param _spec specular color to set
     */
	void setSpecColor(Vect4 _spec)      
    { mat_specColor = _spec;};
    
    /*
     * set the Emissive color of the material
     * @param _emis	emissive color to set
     */
	void setEmisColor(Vect4 _emis)     
    { mat_emisColor = _emis;};
	
	/*
	 * set him as current material
	 */
	void applyMaterial();
	
	/*
	 * set current material as default material (no material)
	 */
	void resetMaterial();
	
	/* 
	 * return if the material is shared
	 * @return true if the material is shared
	 */
    bool isShared();
    
    /*
     * return if the material has diffuse texture
     * @return true if the material has a diffuse texture
     */
    bool hasDiffTex();

	Ega_Texture* getDiffTex(){return mat_diffTex;}
    
    /*
     * return if the material has bump texture
     * @return true if the material has a bump texture
     */
	bool hasBumpTex();

};

#endif
