/*
 * Copyright (C) 2005  ENSI-GAME-ART
 *
 * Club ENSI-GAME-ART ENSICAEN
 * 6 Boulevard Marechal Juin
 * F-14050 Caen Cedex
 *
 * Ce fichier est l'oeuvre d'eleves de l'ENSI de Caen. Il est developp
 * sous la licence GPL :
 *
 ****************************************************************
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *****************************************************************
 *
 * Ce fichier fait partie du projet de jeu video de course spatiale
 * developpe au sein du club ENSI-GAME-ART
 */

#include "engine/Ega_Game.h"
#include "other/Ega_debug.h"


Ega_Game* Ega_Game::gam_this = 0;

void Ega_Game::create()
{
	if (!gam_this)
		gam_this = new Ega_Game();
}

Ega_Game* Ega_Game::getIt()
{
    return gam_this;
}

Ega_Game::Ega_Game()
{
 	gam_win = Ega_Window::getIt();
    gam_scene = Ega_Scene::getIt();
    gam_events = new Ega_GameEventsManager();
    
    init();
	//gam_milEcran.x = win->getWidth()/2;
	//gam_milEcran.y = win->getHeight()/2;
	//SDL_WarpMouse((int)gam_milEcran.x,(int)gam_milEcran.y);
}

Ega_Game::~Ega_Game()
{
}
void Ega_Game::init()
{
	//gam_dzoom=0.0;
	gam_zoom = 150.0;
	gam_dtime = 0.0;
	m_zoom = m_dzoom = m_dpower = m_droll = m_dpitch = 0.0;
}	

bool Ega_Game::updateEvents()
{
    EVENT_TYPE action;
   	
    gam_events->pollAllEvents();
    
    for(int i=0;i<gam_events->getNbEvents();i++)
    {
       action=gam_events->getEvent(i);
       if (action==EVENT_EXIT) return false;
       if (action==EVENT_MOTOR) m_dpower = 100.0; 
       if (action==EVENT_END_MOTOR) m_dpower = -100.0;
	   if (action==EVENT_ZOOM_PLUS) ;
	   if (action==EVENT_ZOOM_MINUS) ;
	   if (action==EVENT_END_ZOOM) ;
	   if (action==EVENT_ROLL_PLUS) m_droll = -2.0; 
	   if (action==EVENT_ROLL_MINUS)m_droll = 2.0; 
	   if (action==EVENT_END_ROLL)  m_droll = 0.0;  
	   if (action==EVENT_PITCH_PLUS) m_dpitch = 2.0;  
	   if (action==EVENT_PITCH_MINUS) m_dpitch = -2.0;  
	   if (action==EVENT_END_PITCH) m_dpitch = 0.0;
    }
    
    gam_player->getShip()->changePowerReactor(m_dpower);
    //gam_player->getShip()->changePowerSide(sin(m_droll/5*2));
    
    gam_player->getShip()->changeAngularSpeed( quaternion(m_droll,Vect3(0,/*-0.6*/0,1.0)) ); 
    gam_player->getShip()->changeAngularSpeed( quaternion(m_dpitch,Vect3(1,0,0)) ); 
    
    return true;
}    

void Ega_Game::play()
{	
   	gam_player = gam_scene->getPlayer();
   	gam_camera = gam_scene->getCamera();
   	gam_lastTime = clock();
   	
   	PRINT_GAME2("pGamePlayer= %p\n", gam_player);
   	PRINT_GAME2("pGameCamera= %p\n", gam_camera);
   	
   	if( gam_camera && gam_player )
   	{
   	
	while( updateEvents() )
	{	
		gam_dtime = (clock() - gam_lastTime)*2;
		gam_lastTime = clock();
		PRINT_GAME1("begin of game loop\n");
		//gam_zoom += 1.0;//gam_dzoom;   		
		gam_scene->makeCycle(gam_dtime); 			
   		
   		gam_camera->setPos(gam_player->getShip()->getPosition()); // position de la camera
   		
   		quaternion _q1,_q2,_q3,_q;

   		_q1 = gam_player->getShip()->getOrientation();
   		_q2 = quaternion(180,Vect3(0,1,0));
   		_q3 = gam_player->getShip()->getAngularSpeed()*0.2;
   		_q = _q1*_q2*_q3;
   		gam_camera->setOrient( _q ); // orientation de la camera
   		
   		Vect3 _v1,_v2,_v;
   		_v1 = Vect3(0.0,50.0,gam_zoom);
   		_v2 = rotate(gam_player->getShip()->getAcceleration() , (_q1*_q2) );
        _v2 *= Vect3(0.001,0.001,0.001);
        _v = _v1 - _v2;
   		gam_camera->setOffset( _v );  //decalage de la cible / camera  !!! ds le repre de la camera
        
        
        
     	gam_win->begin();
     	gam_win->setPerspectiveMode();
   		gam_camera->applyCamera();   	    
   	    
   	    glPushMatrix();
   		gam_scene->draw();
   		glPopMatrix();
   		
   	    
   		
   		gam_win->end();	
   		
   		PRINT_GAME1("end of game loop\n");
	}
	}	
}    
