/*
 * Copyright (C) 2005  ENSI-GAME-ART
 *
 * Club ENSI-GAME-ART ENSICAEN
 * 6 Boulevard Marechal Juin
 * F-14050 Caen Cedex
 *
 * Ce fichier est l'oeuvre d'eleves de l'ENSI de Caen. Il est developp
 * sous la licence GPL :
 *
 ****************************************************************
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *****************************************************************
 *
 * Ce fichier fait partie du projet de jeu video de course spatiale
 * developpe au sein du club ENSI-GAME-ART
 */

#include "engine/Ega_Window.h"

Ega_Window *Ega_Window::win_this = 0;

Ega_Window::Ega_Window(char * WinName, int Width, int Height, bool Fullscreen, bool ShowMouse)
{
    win_this = this;
    win_font = NULL;
	win_width = Width;
	win_height = Height;
	win_fullscreen = Fullscreen;

	SDL_Init(SDL_INIT_EVERYTHING);
	
	
	createWindowContext();
	initGL();
	setFont(TYPO_FONT);   

	if (!ShowMouse)
	{	// Le curseur de la souris devient invisible.
		unsigned char pucData[4] = {0, 0, 0, 0};
		SDL_Cursor* pCursor = SDL_CreateCursor (pucData, pucData, 4, 4, 0, 0);
		SDL_SetCursor (pCursor);
	}

	SDL_WM_SetCaption(WinName, WinName);	
	
	fpsMoy = 0;
	fpsCompte = 0;
	fps = 0;
	curTime = lastTime = clock();
}

Ega_Window::~Ega_Window()
{
    SDL_FreeSurface(win_gl);
    win_this = 0;
    SDL_Quit();
}


void Ega_Window::createWindowContext()
{
	win_info = SDL_GetVideoInfo( );
	if( !win_info ) 
	{     fprintf( stderr, "Echec de la recuperation des infos video: %s\n", SDL_GetError( ) );   }
 
    win_flags = SDL_DOUBLEBUF; 
 	win_flags |= SDL_OPENGL;

    if(win_fullscreen)
        win_flags |= SDL_FULLSCREEN;
    //else    
      //  win_flags |= SDL_NOFRAME;

    if(win_info->hw_available)                      // is it a hardware surface
        win_flags |= SDL_HWSURFACE;
    else
        win_flags |= SDL_SWSURFACE;

    if(win_info->blit_hw)                            // is hardware blitting available
        win_flags |= SDL_HWACCEL;

	SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
	SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 32 );
	SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 5 );
	SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 5 );
	SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 5 );
	SDL_GL_SetAttribute( SDL_GL_ALPHA_SIZE, 5 );
	     
	if ( !(win_gl = SDL_SetVideoMode (win_width, win_height, win_info->vfmt->BitsPerPixel, win_flags)) ){
		fprintf (stderr, "Impossible de crer une fenetre : %s\n", SDL_GetError());
	}  
}

SDL_Surface *Ega_Window::getSDLSurf()
{
        return win_gl;
}

void Ega_Window::begin()
{
	fpsCompte++;
	fps = 1 / (clock() - curTime) * 1000;
	curTime = clock();
	if ( (curTime - lastTime) > 1000.0f)
	{
		lastTime = clock();
		fpsMoy = fpsCompte;
		fpsCompte = 0;
	}

	// On vide les buffer
	glClear (GL_COLOR_BUFFER_BIT |GL_DEPTH_BUFFER_BIT );
	glViewport( 0, 0, win_width, win_height);
}

void Ega_Window::end()
{
	win_fps->setAndDraw(-1, "Fps: %d", fpsMoy);
	SDL_GL_SwapBuffers();
}

void Ega_Window::setFont(char * fontFileName)
{
    if( win_font )
        delete win_font;
        
    win_font = new Ega_Font(fontFileName);
    win_fps = new Ega_GuiLabel(1, 5, 15, win_font);
}

void Ega_Window::setFont(Ega_Font *font)
{
    if( win_font )
        delete win_font;
        
    win_font = font;
    win_fps = new Ega_GuiLabel(1, 5, 15, win_font);
}

void Ega_Window::setPerspectiveMode()
{
    glEnable(GL_DEPTH_TEST);
    glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(70,GLdouble(win_width)/GLdouble(win_height),1,200000);
}
    
void Ega_Window::setOrthoMode()
{
    glDisable(GL_DEPTH_TEST);
    glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(0, GLdouble(win_width), 0, GLdouble(win_height));
}

void Ega_Window::setWindow(int Width, int Height)
{
    win_width = Width;
    win_height = Height;
    createWindowContext();
}    



void Ega_Window::initGL()
{
    glClearColor (0.0f, 0.0f, 0.0f, 1.0f);
	glClear(GL_COLOR_BUFFER_BIT |GL_DEPTH_BUFFER_BIT );
	
	glClearDepth (1.0f);										// Depth Buffer Setup
	glDepthFunc (GL_LESS);									// The Type Of Depth Testing (Less Or Equal)
	glEnable(GL_DEPTH_TEST);									// Enable Depth Testing
	glEnable (GL_NORMALIZE);
	glEnable (GL_LIGHTING);
	glShadeModel (GL_SMOOTH);									// Select Smooth Shading
	glHint (GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);			// Set Perspective Calculations To Most Accurate
	//glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL, GL_SEPARATE_SPECULAR_COLOR);
	glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER,GL_TRUE);
	GLfloat light_ambient[] = { 0.8, 0.8, 0.8, 1.0 };
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, light_ambient);	

	SDL_GL_SwapBuffers ();
}
