/*
 * Copyright (C) 2005  ENSI-GAME-ART
 *
 * Club ENSI-GAME-ART ENSICAEN
 * 6 Boulevard Marechal Juin
 * F-14050 Caen Cedex
 *
 * Ce fichier est l'oeuvre d'eleves de l'ENSI de Caen. Il est developp
 * sous la licence GPL :
 *
 ****************************************************************
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *****************************************************************
 *
 * Ce fichier fait partie du projet de jeu video de course spatiale
 * developpe au sein du club ENSI-GAME-ART
 */
 
#include "engine/Ega_Scene.h"
#include "loader/Ega_Loader.h"
#include "entity/Ega_Entity.h"

#include "other/Ega_debug.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
Ega_Entity::Ega_Entity()
{
    init();
	Objects = new Ega_ObjectManager();
}    

Ega_Entity::Ega_Entity(char *fileName)
{
	init();
	Objects = Ega_Loader::getIt()->loadObjects(fileName);
}

Ega_Entity::Ega_Entity(Ega_ObjectManager *objects)
{
	init();
	Objects = objects;
}

void Ega_Entity::init()
{
    masse = inerty = -1;
	is_physic = false;
	collision_Id = 0;
	
	Vect3 v_init = Vect3(0.0,0.0,0.0);
	speed = v_init;
    acceleration = v_init;
 	up = Vect3(0.0,1.0,0.0);
 	
 	angular_speed = quaternion( 0.0, v_init );
    moment = quaternion( 0.0, v_init );
}

Ega_Entity::~Ega_Entity()
{

}

void Ega_Entity::addObject(Ega_Object *object)
{
    Objects->addElement(object);
}

void Ega_Entity::store()
{
    Ega_Scene::getIt()->addEntity(this);
}    


void Ega_Entity::setName(string _name)
{
    ent_name = _name;
}    

string Ega_Entity::getName()
{
    return ent_name;
}    
	
void Ega_Entity::putPosition()
{
	glTranslatef(position[0],position[1],position[2]);
}

void Ega_Entity::putOrientation()
{
    Vect3 _v = orientation.getAxis();
    float angle = orientation.getAngle();
	glRotatef(angle,_v[0],_v[1],_v[2]);
	//fprintf(stderr, "\n rot entity %d = %lf, %lf, %lf, %lf\n\n", Id, angle, _v[0], _v[1], _v[2]);
}	

void Ega_Entity::putTransform()
{
    PRINT_DISPLAY2("draw entity %d\n", Id);
    glPushMatrix();
 	putPosition();
	glScalef(scale[0],scale[1],scale[2]);
	putOrientation();
}
	
void Ega_Entity::draw(/*bool _bForce*/)
{
    PRINT_DISPLAY2("draw entity %d\n", Id);
	glPushMatrix();
 	putPosition();
	/*if (_bForce)
 	{
 	    glDisable(GL_LIGHTING);
 		glBegin(GL_LINES);
		glColor4f ( 1 , 1 , 1 , 0);
 		glVertex3d(0,0,0);
 		glVertex3d(ent_accel[0]*10,ent_accel[1]*10,ent_accel[2]*10);
 		glEnd();
	}*/
	PRINT_DISPLAY2("scale entity %d\n",scale[0]);
	PRINT_DISPLAY2("             %d\n",scale[1]);
	PRINT_DISPLAY2("             %d\n",scale[2]);
	glScalef(scale[0],scale[1],scale[2]);
	putOrientation();
	
	Objects->drawAll();
	glPopMatrix();
	PRINT_DISPLAY2("end draw entity %d\n", Id);
}

void Ega_Entity::changeScale(Vect3 _dscal)
{
    scale += _dscal;
}
 
void Ega_Entity::changeUp(Vect3 _dup)
{
    up += _dup;
}

void Ega_Entity::setUp(Vect3 _up)
{
    up = _up;
}

void Ega_Entity::changePosition(Vect3 _dpos)
{
 	position += _dpos;
}

void Ega_Entity::changeSpeed(Vect3 _dspeed)
{
	speed += _dspeed;
}    
void Ega_Entity::setSpeed(Vect3 _speed)
{
	speed = _speed;
}    

void Ega_Entity::changeAcceleration(Vect3 _daccel)
{
    acceleration += _daccel;
}    
void Ega_Entity::setAcceleration(Vect3 _accel)
{
    acceleration = _accel;
}

void Ega_Entity::changeOrientation(quaternion & _dorient)
{
    quaternion _temp;
    _temp = (orientation*_dorient);
    orientation = _temp;
} 

void Ega_Entity::changeAngularSpeed(quaternion & _drot)
{
	angular_speed = (angular_speed*_drot);
}    
void Ega_Entity::setAngularSpeed(quaternion & _rot)
{
	angular_speed = _rot;
}    

void Ega_Entity::changeMoment(quaternion & _dmoment)
{
    moment = (moment*_dmoment);
}    
void Ega_Entity::setMoment(quaternion & _moment)
{
    moment = _moment;
}

dGeomID Ega_Entity::getCollisionId()
{
    return collision_Id;
}

bool Ega_Entity::isPhysic()
{
    return is_physic;
}

void Ega_Entity::setPhysic(bool _bphysic)
{
    is_physic = _bphysic;
}

void Ega_Entity::setMasse(int _masse)
{
    masse = _masse;
}

void Ega_Entity::setInerty(int _inert)
{
    inerty = _inert;
} 
