/*
 * Copyright (C) 2005  ENSI-GAME-ART
 *
 * Club ENSI-GAME-ART ENSICAEN
 * 6 Boulevard Marechal Juin
 * F-14050 Caen Cedex
 *
 * Ce fichier est l'oeuvre d'eleves de l'ENSI de Caen. Il est developp
 * sous la licence GPL :
 *
 ****************************************************************
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *****************************************************************
 *
 * Ce fichier fait partie du projet de jeu video de course spatiale
 * developpe au sein du club ENSI-GAME-ART
 */

#ifndef _EGA_ENTITY_H_
#define _EGA_ENTITY_H_

#include "display/Ega_Object.h"
#include "tool/Ega_Tools.h"
#include "manager/Ega_ObjectManager.h"

#include <gl/gl.h>
#include <ode.h>

/**
 * Basic class to manage all entities of the game.
 * All entities (SpaceShips, Asteroids, bonus) inherit of this class
 *
 * @author Gautier Matthieu
 * @author Leduc Pierre
 */
class Ega_Entity: public Ega_Object
{
//friend class Ega_Physic;    

public:
    Ega_Entity();
	Ega_Entity(char *fileName);
	Ega_Entity(Ega_ObjectManager *objects);
	virtual ~Ega_Entity();
	
	virtual void init();
	
	virtual void draw();
	
	virtual void putPosition();
	virtual void putOrientation();
	virtual void putTransform();
	
	virtual void store();
	
	void setName(string _name);
	string getName();
	
    virtual void changeOrientation(quaternion & _dorient);
	virtual void changePosition(Vect3 _dpos);
	virtual void changeScale(Vect3 _dscal);
 
    void changeUp(Vect3 _dup);
	void setUp(Vect3 _up);	
	Vect3 getUp() { return up; };
	
	void changeSpeed(Vect3 _dspeed);
	void setSpeed(Vect3 _speed);
	Vect3 getSpeed() { return speed; };

	void changeAcceleration(Vect3 _daccel);
	void setAcceleration(Vect3 _accel);
	Vect3 getAcceleration() { return acceleration; };
	
	void changeAngularSpeed(quaternion & _drot);
	void setAngularSpeed(quaternion & _rot);
	quaternion getAngularSpeed() { return angular_speed; };

	void changeMoment(quaternion & _dmoment);
	void setMoment(quaternion & _moment);
	quaternion getMoment() { return moment; };

	dGeomID getCollisionId();
	
	bool isPhysic();
	void setPhysic(bool _bphysic);
	void setMasse(int _masse);
	void setInerty(int _inert);
	
	int getMasse(){  return masse;  }
	int getInerty(){  return inerty;  }
	
	void addObject( Ega_Object *object );
	
protected:
    
	Ega_ObjectManager *Objects;
	
	dGeomID collision_Id;
	bool is_physic;
	int masse;
    int inerty;
	
	string ent_name;
	
 	Vect3 speed;
    Vect3 acceleration;
 	Vect3 up;
 	
 	quaternion angular_speed;
    quaternion moment;
};

#endif
