/*
 * Copyright (C) 2005  ENSI-GAME-ART
 *
 * Club ENSI-GAME-ART ENSICAEN
 * 6 Boulevard Marechal Juin
 * F-14050 Caen Cedex
 *
 * Ce fichier est l'oeuvre d'eleves de l'ENSI de Caen. Il est developp
 * sous la licence GPL :
 *
 ****************************************************************
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *****************************************************************
 *
 * Ce fichier fait partie du projet de jeu video de course spatiale
 * developpe au sein du club ENSI-GAME-ART
 */

#include "Ega_ObjectParser.h"
#include "engine/Ega_Scene.h"

Ega_ObjectParser::Ega_ObjectParser()
{

}

Ega_ObjectParser::~Ega_ObjectParser()
{

}

void Ega_ObjectParser::addFace(face _face)
{
	obp_listFaces.push_back(_face);
}

void Ega_ObjectParser::addNormal(Vect3 _normal)
{
	obp_listNormals.push_back(_normal);
}

void Ega_ObjectParser::addVertex(Vect3 _vertex)
{
	obp_listVertex.push_back(_vertex);
}

void Ega_ObjectParser::addTexFace(face _texface)
{
    obp_listTexFaces.push_back(_texface);
}

void Ega_ObjectParser::addTexCoord(Vect2 _texcoord)
{
	obp_listTexCoords.push_back(_texcoord);
}

GLuint Ega_ObjectParser::createDisplayList(Ega_Object *object, Ega_Entity *entity)
{
    int j, n;
    Vect2 texcoord;
    Vect3 normal;
    Vect3 vertex;
    
	GLuint DL = glGenLists(1);
	glNewList(DL, GL_COMPILE);
	
    glPushMatrix();
	Ega_Scene::getIt()->applyMaterial(object->matId);
         
    quaternion rotation = object->getOrientation();
    Vect3 axis = rotation.getAxis();
    glRotatef( -rotation.getAngle(), axis[0], axis[1], axis[2] ); 
    
    Vect3 scale = object->getScale();
	//glScalef(scale[0], scale[1], scale[2]);
    //glScalef( 1.0/scale[0], 1.0/scale[1], 1.0/scale[2]);    
    
    Vect3 translate = Vect3(0,0,0);
    if( entity )
        translate = entity->getPosition(); 
    translate += object->getPosition();
        
    glTranslatef( -translate[0], -translate[1], -translate[2]);
    
    glBegin(GL_TRIANGLES);
	for(j = 0, n = 0; j < obp_listFaces.size(); j++, n+=3)
	{
	    // First Vertex of the face
	    texcoord = obp_listTexCoords[obp_listTexFaces[j].v1];
		glTexCoord2f( texcoord[0], texcoord[1] );
		
		if( obp_listNormals.size() > 0 )
		{
            normal = obp_listNormals[n];
            glNormal3f(normal[0], normal[1], normal[2]);
        }
                
        vertex = obp_listVertex[obp_listFaces[j].v1];
        glVertex3f(vertex[0], vertex[1], vertex[2]);
        
        // Second Vertex of the face
        texcoord = obp_listTexCoords[obp_listTexFaces[j].v2];
		glTexCoord2f( texcoord[0], texcoord[1] );
		
		if( obp_listNormals.size() > 0 )
		{
            normal = obp_listNormals[n+1];
            glNormal3f(normal[0], normal[1], normal[2]);
        }    
         
        vertex = obp_listVertex[obp_listFaces[j].v2];
        glVertex3f(vertex[0], vertex[1], vertex[2]);
  
        // Third Vertex of the face
        texcoord = obp_listTexCoords[obp_listTexFaces[j].v3];
		glTexCoord2f( texcoord[0], texcoord[1] );
		
        if( obp_listNormals.size() > 0 )
		{
            normal = obp_listNormals[n+2];
            glNormal3f(normal[0], normal[1], normal[2]);
        }
            
        vertex = obp_listVertex[obp_listFaces[j].v3];
        glVertex3f(vertex[0], vertex[1], vertex[2]);   
                							
	}
	glEnd();
	
	glPopMatrix();
//	Ega_Scene::getIt()->resetMaterial(object->matId);
	glEndList();
    return DL;            
}
