/*
 * Copyright (C) 2005  ENSI-GAME-ART
 *
 * Club ENSI-GAME-ART ENSICAEN
 * 6 Boulevard Marechal Juin
 * F-14050 Caen Cedex
 *
 * Ce fichier est l'oeuvre d'eleves de l'ENSI de Caen. Il est developp
 * sous la licence GPL :
 *
 ****************************************************************
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *****************************************************************
 *
 * Ce fichier fait partie du projet de jeu video de course spatiale
 * developpe au sein du club ENSI-GAME-ART
 */
 
#include "Ega_Parser.h"

#include "other/Ega_debug.h"
#include <iostream>

Ega_Parser::Ega_Parser(char *fileName)
{		
    fileName_      = fileName;   
    entity_        = NULL;
    object_        = NULL;
    material_      = NULL;
    light_         = NULL;
    camera_        = NULL; 
    objectManager_ = NULL;
}

Ega_Parser::~Ega_Parser()
{
}

int str2Int(char * attrib)
{
        int index ;
        stringstream str ( attrib);
        str >> index ;
        return index;
}

GLuint str2GLuint(char * attrib)
{
        GLuint index ;
        stringstream str ( attrib);
        str >> index ;
        return index;
}

double str2Double(char * attrib)
{
        double index ;
        stringstream str ( attrib);
        str >> index ;
        return index;
}


bool Ega_Parser::parseDocument(){
	TiXmlDocument xmlDocument_ = TiXmlDocument(fileName_);
	if( !xmlDocument_.LoadFile() ) {
        PRINT_LOADER1("Document not parsed successfully. \n");
        return false;
    }
	TiXmlElement *xmlDocumentRootElement = xmlDocument_.FirstChildElement();
	if (xmlDocumentRootElement == NULL) {
        PRINT_LOADER1("empty document\n");
        return false;
    }
	for(;xmlDocumentRootElement;xmlDocumentRootElement = xmlDocumentRootElement->NextSiblingElement()){
		if ( !strcmp(xmlDocumentRootElement->Value(),"egaModel") ){
			if ( !parseEgaModel(xmlDocumentRootElement,false)){
				return false;
			}
		}else{
			cerr << "Unknown value " << xmlDocumentRootElement->Value() << endl;
			return false;
		}
	}
    return true;
}

Ega_ObjectManager* Ega_Parser::parseDocumentObject(){
	objectManager_ = new Ega_ObjectManager();
	TiXmlDocument xmlDocument_ = TiXmlDocument(fileName_);
	if( !xmlDocument_.LoadFile() ) {
        PRINT_LOADER1("Document not parsed successfully. \n");
        return NULL;
    }
	TiXmlElement *xmlDocumentRootElement = xmlDocument_.FirstChildElement();
	if (xmlDocumentRootElement == NULL) {
        PRINT_LOADER1("empty document\n");
        return NULL;
    }
	for(;xmlDocumentRootElement;xmlDocumentRootElement = xmlDocumentRootElement->NextSiblingElement()){
		if ( !strcmp(xmlDocumentRootElement->Value(),"egaModel") ){
			if ( !parseEgaModel(xmlDocumentRootElement,true)){
				return NULL;
			}
		}else{
			cerr << "Unknown value " << xmlDocumentRootElement->Value() << endl;
			return NULL;
		}
	}
    return objectManager_;
}

bool Ega_Parser::parseEgaModel(TiXmlElement* mainElement,bool onlyGeom){
	TiXmlElement* currentElement = mainElement->FirstChildElement();
	for(;currentElement;currentElement = currentElement->NextSiblingElement()){
		if( !strcmp((char*)currentElement->Value() , "scLf") ){
				continue;
				// what is scLf ??? Nothing doing in the last code, so nothing is doing here.
			}else if ( !strcmp(currentElement->Value() , "scFs") ){
				continue;
				// what is scFs ??? Nothing doing in the last code, so nothing is doing here.
			}else if ( !strcmp(currentElement->Value() , "scTpf") ){
				continue;
				// what is scTpf ??? Nothing doing in the last code, so nothing is doing here.
			}
		if (!onlyGeom){
			if ( !strcmp(currentElement->Value() , "grp") ){
				if ( !parseGroup(currentElement))
					return false;
				continue;
			}else if ( !strcmp(currentElement->Value() , "liObj") ){
				if ( !parseLight(currentElement))
					return false;
				continue;
			}else if( !strcmp(currentElement->Value() , "caObj") ){
				if ( !parseCamera(currentElement))
					return false;
				continue;
			}
		}
		if ( !strcmp(currentElement->Value() , "maId") ){
			if ( !parseMaterial(currentElement))
				return false;
			continue;
		}else if ( !strcmp(currentElement->Value() , "obj") ){
			if ( !parseObject(currentElement, (onlyGeom?2:0)))
				return false;
			continue;
		}else{
			cerr << "Unknown value " << currentElement->Value() << endl;
			return false;
		}
	}
	return true;
}

bool Ega_Parser::parseMaterial(TiXmlElement* mainElement){
	double a,b,c;
	int i;
	char* difMapPath;
	char* opaMapPath;
	material_ = new Vcb_Material();
	
	TiXmlHandle mainHandle(mainElement);
	TiXmlElement* currentElement;

	currentElement = mainHandle.FirstChildElement("maDif").ToElement();;
	if(currentElement){
		currentElement->Attribute("a0",&a);	currentElement->Attribute("a1",&b);	currentElement->Attribute("a2",&c);
		material_->setDiffColor(Vect4(a, b, c, 1.0));
	}
	
	currentElement = mainHandle.FirstChildElement("maAmb").ToElement();
	if(currentElement){
		currentElement->Attribute("a0",&a);	currentElement->Attribute("a1",&b);	currentElement->Attribute("a2",&c);
		material_->setAmbientColor(Vect4(a, b, c, 1.0));
	}

	currentElement = mainHandle.FirstChildElement("maSpec").ToElement();
	if(currentElement){
		currentElement->Attribute("a0",&a);	currentElement->Attribute("a1",&b);	currentElement->Attribute("a2",&c);
		material_->setSpecColor(Vect4(a, b, c, 1.0));
	}

	currentElement = mainHandle.FirstChildElement("maShi").ToElement();
	if(currentElement){
		currentElement->Attribute("a0",&a);
		material_->setShininess(100.0 * a);
	}

	currentElement = mainHandle.FirstChildElement("maOpa").ToElement();
	if(currentElement){
		currentElement->Attribute("a0",&a);
		material_->setOpacity(1.0 - a);
	}

	currentElement = mainHandle.FirstChildElement("mapDif").FirstChildElement("bmp").ToElement();
	if(currentElement){
		difMapPath = (char*)currentElement->Attribute("a0");
		currentElement = mainHandle.FirstChildElement("mapOpac").FirstChildElement("bmp").ToElement();
		if(currentElement){
			opaMapPath = (char*)currentElement->Attribute("a0");
			material_->setDiffTexture(difMapPath, opaMapPath);
		}else
			material_->setDiffTexture(difMapPath);
	}

	mainElement->Attribute("i",&i);
	tempMapMaterialId_[i] =  material_->store();

	material_ = NULL;

	return true;
}
/**
 * Ega_parser::parseObject
 * Charge une geomtrie;
 * mainElement: noeud xml correspondant  la racine de la geometrie
 * objectMode: Faon de charger la geometrie : 0 nouvelle entity, 1 entity dj cre, 2 pas d'entity
 */
bool Ega_Parser::parseObject(TiXmlElement* mainElement, int objectMode){
	double a,b,c;
	int i;
	if(objectMode == 0)
		entity_ = new Ega_Entity();
    object_ = new Ega_Object();
    objParser_ = new Ega_ObjectParser();
	TiXmlElement* currentElement = mainElement->FirstChildElement();
	for(;currentElement;currentElement = currentElement->NextSiblingElement()){
		if( !strcmp(currentElement->Value() , "maRef") ){
			currentElement->Attribute("i",&i);
			object_->setMaterialID(tempMapMaterialId_[i]);
		}else if ( !strcmp(currentElement->Value() , "mhVer") ){
			currentElement->Attribute("a0",&a);
			currentElement->Attribute("a1",&b);
			currentElement->Attribute("a2",&c);
			objParser_->addVertex(Vect3( a, b, c ));
		}else if ( !strcmp(currentElement->Value() , "mhFac") ){
			currentElement->Attribute("a0",&a);
			currentElement->Attribute("a1",&b);
			currentElement->Attribute("a2",&c);
			objParser_->addFace(face( a, b, c ));
		}else if ( !strcmp(currentElement->Value() , "mhNorV") ){
			currentElement->Attribute("a0",&a);
			currentElement->Attribute("a1",&b);
			currentElement->Attribute("a2",&c);
			objParser_->addNormal(Vect3( a, b, c ));
		}else if ( !strcmp(currentElement->Value() , "mhTVer") ){
			currentElement->Attribute("a0",&a);
			currentElement->Attribute("a1",&b);
			objParser_->addTexCoord(Vect2( a, b));
		}else if ( !strcmp(currentElement->Value() , "mhTFac") ){
			currentElement->Attribute("a0",&a);
			currentElement->Attribute("a1",&b);
			currentElement->Attribute("a2",&c);
			objParser_->addTexFace(face( a, b, c ));
		}else if( !strcmp(currentElement->Value() , "pos") ){
			currentElement->Attribute("a0",&a);
			currentElement->Attribute("a1",&b);
			currentElement->Attribute("a2",&c);
			if(objectMode != 0)
				object_->setPosition(Vect3( a, b, c));
			else
				entity_->setPosition(Vect3( a, b, c));
		}else if( !strcmp(currentElement->Value() , "rotax") ){
			currentElement->Attribute("a0",&a);
			currentElement->Attribute("a1",&b);
			currentElement->Attribute("a2",&c);
			object_->setOrientAxis(Vect3( a, b, c));
		}else if( !strcmp(currentElement->Value() , "rotang") ){
			currentElement->Attribute("a0",&a);
			object_->setOrientAngle(a);	
		}else if( !strcmp(currentElement->Value() , "scal") ){
			currentElement->Attribute("a0",&a);
			currentElement->Attribute("a1",&b);
			currentElement->Attribute("a2",&c);
			object_->setScale(Vect3( a, b, c));
		}else{
			cerr << "Unknown value " << currentElement->Value() << endl;
			return false;
		}
	}
	
	if(objectMode == 0){
		object_->setDisplayList( objParser_->createDisplayList(object_, entity_) );
		entity_->addObject( object_ );
		entity_->store();
		entity_ = NULL;
	}else if(objectMode ==1){
		object_->setDisplayList( objParser_->createDisplayList(object_, entity_) );
		entity_->addObject( object_ );
	}else{
		object_->setDisplayList( objParser_->createDisplayList(object_, NULL) );
		objectManager_->addElement(object_);
	}
    PRINT_LOADER1("perfect load Object2\n"); 
	return true;
}
bool Ega_Parser::parseGroup(TiXmlElement* mainElement){
	double a,b,c;
	entity_ = new Ega_Entity();
	TiXmlElement* currentElement = mainElement->FirstChildElement();
	for(;currentElement;currentElement = currentElement->NextSiblingElement()){
		if ( !strcmp(currentElement->Value() , "obj") ){
			if ( !parseObject(currentElement,1))
				return false;
		}else if( !strcmp(currentElement->Value() , "pos") ){
			currentElement->Attribute("a0",&a);
			currentElement->Attribute("a1",&b);
			currentElement->Attribute("a2",&c);
			entity_->setPosition(Vect3( a, b, c));
		}else if( !strcmp(currentElement->Value() , "rotax") ){
			currentElement->Attribute("a0",&a);
			currentElement->Attribute("a1",&b);
			currentElement->Attribute("a2",&c);
			entity_->setOrientAxis(Vect3( a, b, c));
		}else if( !strcmp(currentElement->Value() , "rotang") ){
			currentElement->Attribute("a0",&a);
			entity_->setOrientAngle(a);
		}else if( !strcmp(currentElement->Value() , "scal") ){
			currentElement->Attribute("a0",&a);
			currentElement->Attribute("a1",&b);
			currentElement->Attribute("a2",&c);
			entity_->setScale(Vect3( a, b, c));
		}else{
			cerr << "Unknown value " << currentElement->Value() << endl;
			return false;
		}
	}
	entity_->store();
	entity_ = NULL;
	return true;
}
bool Ega_Parser::parseLight(TiXmlElement* mainElement){
	double a,b,c;
	light_ = new Vcb_Light();
	TiXmlHandle mainHandle(mainElement);
	TiXmlElement* currentElement;

	currentElement = mainHandle.FirstChildElement("liCol").ToElement();
	if(currentElement){
		currentElement->Attribute("a0",&a);	currentElement->Attribute("a1",&b);	currentElement->Attribute("a2",&c);
		light_->setDiffColor(Vect4(a, b, c, 1.0));
	}

	currentElement = mainHandle.FirstChildElement("pos").ToElement();
	if(currentElement){
		currentElement->Attribute("a0",&a);	currentElement->Attribute("a1",&b);	currentElement->Attribute("a2",&c);
		light_->setPos(Vect4(a, b, c, 1.0));
	}

	light_->store();
	light_ = NULL;
	return true;
}
bool Ega_Parser::parseCamera(TiXmlElement* mainElement){
	double a,b,c;
	camera_ = new Ega_Camera();
	TiXmlHandle mainHandle(mainElement);
	TiXmlElement* currentElement;

	currentElement = mainHandle.FirstChildElement("caNear").ToElement();
	if(currentElement){
		currentElement->Attribute("a0",&a);
		camera_->setNearPlan(a);
	}

	currentElement = mainHandle.FirstChildElement("caFovy").ToElement();
	if(currentElement){
		currentElement->Attribute("a0",&a);
		camera_->setFovy(a);
	}

	currentElement = mainHandle.FirstChildElement("caFovy").ToElement();
	if(currentElement){
		currentElement->Attribute("a0",&a);	currentElement->Attribute("a1",&b);	currentElement->Attribute("a2",&c);
		camera_->setPos(Vect3(a, b, c));
	}

	camera_->store();
	camera_ = NULL;
	return true;
}
