/*
 * Copyright (C) 2005  ENSI-GAME-ART
 *
 * Club ENSI-GAME-ART ENSICAEN
 * 6 Boulevard Marechal Juin
 * F-14050 Caen Cedex
 *
 * Ce fichier est l'oeuvre d'eleves de l'ENSI de Caen. Il est developp
 * sous la licence GPL :
 *
 ****************************************************************
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *****************************************************************
 *
 * Ce fichier fait partie du projet de jeu video de course spatiale
 * developpe au sein du club ENSI-GAME-ART
 */

#ifndef _EGA_PARSER_H_
#define _EGA_PARSER_H_

#include "engine/Ega_Scene.h"
#include "tool/Ega_Tools.h"
#include "Ega_ParserTags.h"
#include "Ega_ObjectParser.h"
#include "display/Ega_Object.h"
#include "entity/Ega_Entity.h"
#include "display/Vcb_Light.h"
#include "other/Ega_Camera.h"
#include <sstream>

#include <tinyxml.h>

#define SCENE    0
#define ELEMENTS 1
#define OBJECTS  2

using namespace std ;

/**
 * Parse a xml-file.
 *
 * @author Leduc Pierre
 */
class Ega_Parser 
{
friend class Ega_ObjectParser;
public:
        Ega_Parser( char* fileName );
        virtual ~Ega_Parser();
        
        bool getCamera();
        bool getLight();
        bool getObject();
        bool getGroupObject();
        bool getTransform();
        bool getMaterial();
        char *getMaterialMap();
        
        bool getSceneElements();
        bool getObjectElements();
        
        bool initParse();        
        //bool ParseElements();
		bool parseDocument();
		Ega_ObjectManager* parseDocumentObject();
        /*Ega_ObjectManager* ParseObjects();*/
        
        int    getIndice();
        int    getIntAttrib(char* type);
        GLuint getGLuintAttrib(char* type);
        double getDoubleAttrib(char* type);
        char*  getStringAttrib();
        bool   isFlag(char * flag);

private:

		
		bool parseEgaModel(TiXmlElement* mainElement,bool objectOnly);
		bool parseMaterial(TiXmlElement* mainElement);
		bool parseObject(TiXmlElement* mainElement,int objectMode);
		bool parseGroup(TiXmlElement* mainElement);
		bool parseLight(TiXmlElement* mainElement);
		bool parseCamera(TiXmlElement* mainElement);
    
        char              *fileName_;      
        
        Ega_ObjectParser  *objParser_;        
        Ega_Entity        *entity_;
        Ega_Object        *object_;        
        Vcb_Material      *material_;
        Vcb_Light         *light_;
        Ega_Camera        *camera_;
        Ega_ObjectManager *objectManager_;
        
        
        GLuint             tempMapMaterialId_[20];
        GLuint             par_matIdActiv;       
        char               par_matDiffTexPath[180];
        char               par_matOpacTexPath[180];
        char               par_matBumpTexPath[180];
};

#endif
