/*
 * Copyright (C) 2005  ENSI-GAME-ART
 *
 * Club ENSI-GAME-ART ENSICAEN
 * 6 Boulevard Marechal Juin
 * F-14050 Caen Cedex
 *
 * Ce fichier est l'oeuvre d'eleves de l'ENSI de Caen. Il est developp
 * sous la licence GPL :
 *
 ****************************************************************
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *****************************************************************
 *
 * Ce fichier fait partie du projet de jeu video de course spatiale
 * developpe au sein du club ENSI-GAME-ART
 */


#include "tool/Ega_Quaternion.h"

#include <math.h>

quaternion::quaternion()
{
    m_val = 0.0;
    m_axe = Vect3(0,0,0);
}

quaternion::quaternion(Vect3 _axe)
{
    m_val = 0.0;
    m_axe = _axe;
}
    
quaternion::quaternion(float _angle, Vect3 _axe)
{
    if ( ((_angle<=0.00001) && (_angle>=-0.00001)) || (_axe==Vect3(0,0,0)) )
    {
        m_val = 1;
        m_axe = Vect3(0,0,0);
    }
    else
    {    
        m_val = cosf(_angle*M_PI/360);
        m_axe = _axe;  
        m_axe *= sinf(_angle*M_PI/360);
        normalise();
    }    
}

quaternion::quaternion(Vect3 _orig, Vect3 _dest)
{
    _orig.normalise();
	_dest.normalise(); 
                                                                
    if ( (_orig==Vect3(0,0,0)) || (_dest==Vect3(0,0,0)) )
    {
        m_val = 1;
        m_axe = Vect3(0,0,0);
    }else
    {
        m_axe = (( _orig )^( _dest )).normalized();
        float _angle = ( _orig%_dest );
        m_val =  sqrtf((1+_angle)/2);
        m_axe *= sqrtf((1-_angle)/2);
    }    
}    

quaternion::quaternion(const quaternion &_quat)
{
    m_val = _quat.m_val;
    m_axe = _quat.m_axe;
}
    
void quaternion::setAngle(float _angle)
{
    Vect3 _taxe = getAxis();
    *this = quaternion(_angle,_taxe);
}

void quaternion::setAxis(Vect3 _axe)
{
    float _tangle = getAngle();
    *this = quaternion(_tangle,_axe);
}

void quaternion::changeAngle(float _dangle)
{
    *this = *this * quaternion(_dangle,m_axe);
}
    
quaternion quaternion::operator+(quaternion _q)
{
	quaternion temp;
    temp.m_val = m_val + _q.m_val;
    temp.m_axe = m_axe + _q.m_axe;
	return temp;
}
    
quaternion quaternion::operator-(quaternion _q)
{
    quaternion temp;
    temp.m_val = m_val - _q.m_val;
    temp.m_axe = m_axe - _q.m_axe;
	return temp;
}

quaternion quaternion::operator*(quaternion _q)
{
	quaternion temp;
	if ( (abs(_q.m_val-1)<=0.00001) && (_q.m_axe==Vect3(0,0,0)) )
	{
	    temp = *this;
    }else
    {
        if ( (abs(m_val-1)<=0.00001) && (m_axe==Vect3(0,0,0)) )
        {
            temp = _q;
        }else
        {
	        temp.m_val = m_val*_q.m_val - m_axe%_q.m_axe;
	        temp.m_axe = ((_q.m_axe*m_val) + (m_axe*_q.m_val) + (m_axe^_q.m_axe));
	        temp.normalise();
	    }
    }          
	return temp;
}

quaternion quaternion::operator*(float _f)
{
    float _angle = getAngle();
    Vect3 _axe(getAxis());
    _angle *= _f;
    return quaternion(_angle,_axe);
}


quaternion quaternion::operator~()
{
    quaternion temp;
    temp.m_val = m_val;
    temp.m_axe = -m_axe;
    return temp;
}    

quaternion quaternion::operator=(quaternion _q)
{
    m_val = _q.m_val;
    m_axe = _q.m_axe;
    return *this;
}    

float quaternion::getAngle()
{
    return 360*acosf(m_val)/M_PI;
}
    
Vect3 quaternion::getAxis()
{
    if (m_axe == Vect3(0,0,0))
        return Vect3(0,1,0);
        
    float n2 = sqrtf(m_axe[0]*m_axe[0]+m_axe[1]*m_axe[1]+m_axe[2]*m_axe[2]);
    
    if ( n2 < 0.0005 ) 
    n2 = 1;
    
    return m_axe/n2;
}

void quaternion::normalise()
{
    float norme = sqrtf(m_val*m_val+m_axe[0]*m_axe[0]+m_axe[1]*m_axe[1]+m_axe[2]*m_axe[2]);
    if (norme>0.00001)
    {
        m_val /= norme;
        m_axe /= norme;
    }else
    {
        m_val = 1;
        m_axe = Vect3(0,0,0);
    }        
}    


Vect3 rotate(Vect3 _v3,quaternion _q)
{
    quaternion _temp,_result,_result1,_qm1;
    float _norme;

    _norme = _v3.norme();    
    _temp.m_val = 0;
    _temp.m_axe = _v3.normalized();

    
    _qm1.m_val = _q.m_val;
    _qm1.m_axe = -_q.m_axe;

	_result = _q*_temp*_qm1;
    return _result.m_axe*_norme;
}    
